/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package pp4u;

import pp4u_packet.Packet;
import pp4u_packet.PacketPing;
import pp4u_packet.PacketPong;
import pp4u_packet.PacketQuery;
import pp4u_packet.PacketFound;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.Socket;
import java.util.Comparator;
import java.util.List;
import pp4u_manage.ManageKeys;
import pp4u_manage.ManageLinks;
import pp4u_manage.ManageNodes;
import pp4u_packet.PacketQuit;

/**
 * 検索リンク
 * @author tondol
 */
public class LinkSearch implements Runnable {

    private Socket socket = null;
    private ObjectInputStream in = null;
    private ObjectOutputStream out = null;
    private volatile int priority = -1;
    private volatile Thread thread = null;
    private volatile boolean behind = false; // NATを噛ましているかどうか
    private volatile InfoNode gaddr = null; // 接続先のグローバルアドレス
    private volatile InfoNode paddr = null; // 接続先のプライベートアドレス
    private volatile String nodeID = null; // 接続先のノードID
    private volatile String linkID = null; // リンクID

    /**
     * 接続側のリンクを生成
     */
    public LinkSearch(InfoNode node) throws Exception {
        try {
            this.socket = new Socket(node.getHostname(), node.getPort());
            this.out = new ObjectOutputStream(socket.getOutputStream());
            this.out.flush();
            PacketPing ping = new PacketPing(PacketPing.SEARCH);
            send(ping);
            this.in = new ObjectInputStream(socket.getInputStream());
            PacketPong pong = (PacketPong) recv();

            String ghost = socket.getInetAddress().getHostAddress();
            int gport = pong.getAddress().getPort();
            this.paddr = pong.getAddress();
            this.gaddr = new InfoNode(ghost, gport);
            if (!paddr.getHostname().equals(gaddr.getHostname())) {
                this.behind = true;
            }
            this.nodeID = pong.getNodeID();
            this.linkID = ping.getLinkID();

            this.thread = new Thread(this);
            this.thread.start();
            this.priority = 0;
        } catch (Exception ex) {
            stop();
            throw ex;
        }
    }

    /**
     * 受信側のリンクを生成 - pingは受信済み
     */
    public LinkSearch(Socket socket, ObjectInputStream in, PacketPing ping) throws Exception {
        try {
            this.socket = socket;
            this.in = in;
            this.out = new ObjectOutputStream(socket.getOutputStream());
            this.out.flush();
            PacketPong pong = new PacketPong(PacketPong.OK);
            out.writeObject(pong);

            String ghost = socket.getInetAddress().getHostAddress();
            int gport = ping.getAddress().getPort();
            this.paddr = ping.getAddress();
            this.gaddr = new InfoNode(ghost, gport);
            if (!paddr.getHostname().equals(gaddr.getHostname())) {
                this.behind = true;
            }
            this.nodeID = ping.getNodeID();
            this.linkID = ping.getLinkID();

            this.thread = new Thread(this);
            this.thread.start();
            this.priority = 0;
        } catch (Exception ex) {
            stop();
            throw ex;
        }
    }

    /**
     * 接続停止
     */
    public void stop() {
        if (thread != null) {
            thread.interrupt();
        }
        if (socket != null) {
            try {
                send(new PacketQuit());
            } catch (Exception ex) {
            }
            try {
                socket.close();
            } catch (Exception ex) {
            }
        }
    }

    /**
     * 受信スレッド
     */
    public void run() {
        try {
            while (!socket.isClosed()) {
                Packet packet = recv();
                switch (packet.getCommand()) {
                    case Packet.QUERY:
                        PacketQuery query = (PacketQuery) packet;
                        callQuery(query);
                        break;
                    case Packet.FOUND:
                        PacketFound found = (PacketFound) packet;
                        callFound(found);
                        break;
                    case Packet.QUIT:
                        stop();
                        break;
                }
            }
        } catch (Exception ex) {
            if (socket != null) {
                try {
                    send(new PacketQuit());
                } catch (IOException iex) {
                }
                try {
                    socket.close();
                } catch (IOException iex) {
                }
            }
        } finally {
            thread = null;
            socket = null;
            priority = -1;
        }
    }

    /**
     * QUERY受信時の処理
     */
    private void callQuery(PacketQuery query) {
        ManageLinks linkManager = ManageLinks.getInstance();
        ManageKeys keyManager = ManageKeys.getInstance();
        String localID = AppMain.getNodeID();

        // 深さが5以上なら廃棄
        if (query.getPath().size() > 5) {
            return;
        }
        // すでに到達してたら廃棄
        if (query.getPath().contains(localID)) {
            return;
        }

        // 検索
        List<InfoKey> matchedList = keyManager.searchLocal(query.getWord());
        if (!matchedList.isEmpty()) {
            PacketFound found = new PacketFound(matchedList, query.getPath());
            try {
                found.pop();
                send(found);
            } catch (IOException iex) {
            }
        }
        // 中継する
        try {
            query.push(localID);
            linkManager.broadcast(query, this);
        } catch (IOException iex) {
        }
    }

    /**
     * FOUND受信時の処理
     */
    private void callFound(PacketFound found) {
        ManageLinks linkManager = ManageLinks.getInstance();
        ManageKeys keyManager = ManageKeys.getInstance();

        if (found.getPath().size() == 0) {
            // キー受信 - リンク追加は未実装
            for (InfoKey key : found.getKeys()) {
                key.setLocalKey(false);
                keyManager.add(key);
            }
        } else {
            // リンク切れをチェック
            String nextID = found.pop();
            LinkSearch node = linkManager.search(nextID);
            if (node == null) {
                return;
            }
            // 中継する
            try {
                node.send(found);
            } catch (IOException iex) {
            }
        }
    }

    /**
     * このノードにパケットを送信
     */
    public void send(Packet packet) throws IOException {
        out.writeObject(packet);
        out.reset();
    }

    /**
     * このノードからパケットを受信
     */
    public Packet recv() throws IOException, ClassNotFoundException {
        return (Packet) in.readObject();
    }

    //////////////////////////////////////////////
    // 取得・設定メソッド
    //////////////////////////////////////////////
    /**
     * 優先度を取得
     */
    public int getPriority() {
        return priority;
    }

    /**
     * グローバルアドレスを返す
     */
    public InfoNode getGrobalAddr() {
        return gaddr;
    }

    /**
     * プライベートアドレスを返す
     */
    public InfoNode getPrivateAddr() {
        return paddr;
    }

    /**
     * ノードIDを返す
     */
    public String getNodeID() {
        return nodeID;
    }

    /**
     * リンクIDを返す
     */
    public String getLinkID() {
        return linkID;
    }

    /**
     * NATの後ろにあるかどうか
     */
    public boolean isBehind() {
        return behind;
    }

    //////////////////////////////////////////////
    // インターフェイスの実装
    //////////////////////////////////////////////
    /**
     * リンクIDで比較
     */
    @Override
    public boolean equals(Object obj) {
        if (obj == null || getClass() != obj.getClass()) {
            return false;
        }
        final LinkSearch other = (LinkSearch) obj;
        if (!linkID.equals(other.linkID)) {
            return false;
        }
        return true;
    }

    /**
     * ノードIDでハッシュ計算
     */
    @Override
    public int hashCode() {
        return linkID.hashCode();
    }
}
