/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package pp4u;

import java.io.EOFException;
import java.io.File;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.RandomAccessFile;
import java.net.Socket;
import pp4u_packet.Packet;
import pp4u_packet.PacketGet;
import pp4u_packet.PacketPing;
import pp4u_packet.PacketPong;
import pp4u_packet.PacketPut;

/**
 * 送信タスク
 * @author tondol
 */
public class TaskUpload extends TaskAbstract implements Runnable {

    private Socket socket = null;
    private ObjectInputStream in = null;
    private ObjectOutputStream out = null;
    private volatile InfoNode gaddr = null;
    private volatile InfoNode paddr = null;
    private volatile Thread thread = null;
    private volatile InfoKey key = null;
    private volatile long current = 0;
    private volatile long length = 0;

    public TaskUpload(Socket socket, ObjectInputStream in, PacketPing ping) throws Exception {
        try {
            this.socket = socket;
            this.in = in;
            this.out = new ObjectOutputStream(socket.getOutputStream());
            this.out.flush();
            PacketPong pong = new PacketPong(PacketPong.OK);
            out.writeObject(pong);

            String ghost = socket.getInetAddress().getHostAddress();
            int gport = pong.getAddress().getPort();
            this.paddr = pong.getAddress();
            this.gaddr = new InfoNode(ghost, gport);

            thread = new Thread(this);
            thread.start();
        } catch (Exception ex) {
            stop();
            throw ex;
        }
    }

    public void stop() {
        if (thread != null) {
            thread.interrupt();
        }
    }

    public void run() {
        RandomAccessFile ist = null;
        byte[] buffer = new byte[65536];
        try {
            while (!thread.isInterrupted()) {
                // GETコマンドを受信
                PacketGet get = (PacketGet) recv();
                if (key == null) {
                    this.key = get.getKey();
                    this.length = key.getSize();
                    File input = new File(key.getUploadPath());
                    ist = new RandomAccessFile(input, "r");
                }
                // ポインタとか設定
                this.current = get.getIndex() * 65536;
                int loaded = (int) Math.min(length - current, 65536);
                ist.seek(current);
                ist.readFully(buffer, 0, loaded);
                PacketPut put = new PacketPut(key, get.getIndex(), buffer);
                send(put);
            }
        } catch (EOFException eex) {
            // 正常終了時でも発生，原因不明
        } catch (Exception ex) {
            AppMain.debug("# 送信タスクエラー");
            AppMain.debug("@ " + ex);
        } finally {
            try {
                if (ist != null) {
                    ist.close();
                }
            } catch (IOException iex) {
            }
            try {
                if (socket != null) {
                    socket.close();
                }
            } catch (IOException iex) {
            } finally {
                thread = null;
            }
        }
    }

    private void send(Packet packet) throws IOException {
        out.writeObject(packet);
        out.reset();
    }

    private Packet recv() throws IOException, ClassNotFoundException {
        return (Packet) in.readObject();
    }

    //////////////////////////////////////////////
    // 取得・設定メソッド
    //////////////////////////////////////////////
    @Override
    public boolean isAlive() {
        if (thread == null) {
            return false;
        } else {
            return true;
        }
    }

    @Override
    public String getType() {
        return "送信タスク";
    }

    @Override
    public String getFilename() {
        if (key == null) {
            return null;
        } else {
            return key.getFilename();
        }
    }

    @Override
    public InfoNode getNode() {
        if (gaddr == null) {
            return null;
        } else {
            return gaddr;
        }
    }

    @Override
    public String getStatus() {
        int per = (int) (100.0 * (double) current / (double) length);
        return per + "% @ " + current + "/" + length;
    }

    //////////////////////////////////////////////
    // インターフェイスの実装
    //////////////////////////////////////////////
    @Override
    public boolean equals(Object obj) {
        if (obj == null || getClass() != obj.getClass()) {
            return false;
        }
        final TaskUpload other = (TaskUpload) obj;
        if (!this.key.equals(other.key)) {
            return false;
        }
        return true;
    }

    @Override
    public int hashCode() {
        return key.hashCode();
    }
}
