/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package pp4u_manage;

import java.io.EOFException;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FilenameFilter;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.ArrayList;
import java.util.List;
import pp4u.*;

/**
 * キー管理クラス - startでスキャン開始, stopで仮想キークリア
 * @author tondol
 */
public class ManageKeys implements Runnable {

    static private volatile ManageKeys instance = null;
    private volatile ArrayList<InfoKey> keys;
    private volatile Thread thread = null;

    private ManageKeys() {
        keys = new ArrayList<InfoKey>();
    }

    /**
     * インスタンス化
     */
    static synchronized public ManageKeys getInstance() {
        if (instance == null) {
            instance = new ManageKeys();
        }
        return instance;
    }

    /**
     * ローカルキーを読み込む
     */
    public synchronized void load() {
        ObjectInputStream ist = null;
        FileInputStream fst = null;

        try {
            fst = new FileInputStream("pp4ukey.db");
            ist = new ObjectInputStream(fst);
            Object obj = null;
            while ((obj = ist.readObject()) != null) {
                add((InfoKey) obj);
            }
            ist.close();
        } catch (EOFException eex) {
            AppMain.debug("O キーリスト読込成功");
        } catch (Exception ex) {
            AppMain.debug("X キーリスト読込失敗");
            AppMain.debug("@ " + ex);
        }
    }

    /**
     * スキャン開始
     */
    public synchronized void start() {
        if (thread == null) {
            thread = new Thread(this);
            thread.start();
            BoxMain.getInstance().setScan(true);
            BoxMain.getInstance().applyState();
        }
    }

    /**
     * キー後処理
     */
    public synchronized void stop() {
        try {
            // ファイルを開く
            FileOutputStream fst = new FileOutputStream("pp4ukey.db");
            ObjectOutputStream ost = new ObjectOutputStream(fst);
            InfoKey[] arrayKeys = null;
            arrayKeys = keys.toArray(new InfoKey[0]);

            // キーリストを書き込む
            for (InfoKey key : arrayKeys) {
                if (key.isLocalKey()) {
                    ost.writeObject(key);
                } else {
                    remove(key);
                }
            }
            ost.flush();
            ost.close();

            // デバッグ表示
            AppMain.debug("O キーリスト書込成功");
        } catch (Exception ex) {
            AppMain.debug("X キーリスト書込失敗");
            AppMain.debug("@ " + ex);
        } finally {
            if (thread != null) {
                thread.interrupt();
            }
        }
    }

    /**
     * ローカルキーを更新
     */
    public void run() {
        try {
            // フォルダ内のファイルのみを取得
            InfoKey[] arrayKeys = null;
            File directory = ManageConfigure.getInstance().getUpload();
            File[] arrayFiles = directory.listFiles(new FilenameFilter() {

                public boolean accept(File path, String name) {
                    return (new File(path + File.separator + name)).isFile();
                }
            });

            // 追加
            arrayKeys = searchLocal("").toArray(new InfoKey[0]);
            for (File file : arrayFiles) {
                boolean flag = true;
                for (InfoKey key : arrayKeys) {
                    if (key.getFilename().equals(file.getName())) {
                        flag = false;
                        break;
                    }
                }
                if (flag) {
                    add(new InfoKey(file));
                }
            }
            // 削除
            arrayKeys = searchLocal("").toArray(new InfoKey[0]);
            for (InfoKey key : arrayKeys) {
                boolean flag = true;
                for (File file : arrayFiles) {
                    if (file.getName().equals(key.getFilename())) {
                        flag = false;
                        break;
                    }
                }
                if (flag) {
                    remove(key);
                }
            }
            // 更新
            arrayKeys = searchLocal("").toArray(new InfoKey[0]);
            for (InfoKey key : arrayKeys) {
                File file = new File(key.getUploadPath());
                if (key.getModified() != file.lastModified()) {
                    synchronized (keys) {
                        int index = keys.indexOf(key);
                        keys.set(index, new InfoKey(file));
                    }
                } else if (key.getNode() == null) {
                    key.setNode(AppMain.getLocalhost());
                } else if (!key.getNode().equals(AppMain.getLocalhost())) {
                    key.setNode(AppMain.getLocalhost());
                }
            }

            // デバッグ表示
            AppMain.debug("O スキャン成功");
        } catch (Exception ex) {
            AppMain.debug("X スキャン失敗");
            AppMain.debug("@ " + ex);
        } finally {
            thread = null;
            BoxMain.getInstance().setScan(false);
            BoxMain.getInstance().applyState();
        }
    }

    /**
     * ローカルキーを検索
     */
    public List<InfoKey> searchLocal(String word) {
        ArrayList<InfoKey> list = null;
        list = new ArrayList<InfoKey>();
        for (InfoKey key : keys) {
            if (key.isLocalKey() && key.matches(word)) {
                list.add(key);
            }
        }
        return list;
    }

    /**
     * キー全体から検索
     */
    public List<InfoKey> searchAll(String word) {
        ArrayList<InfoKey> list = null;
        list = new ArrayList<InfoKey>();
        for (InfoKey key : keys) {
            if (key.matches(word)) {
                list.add(key);
            }
        }
        return list;
    }

    public int getSize() {
        return keys.size();
    }

    /**
     * キーを追加
     */
    public void add(InfoKey key) {
        if (keys.contains(key)) {
            return;
        }
        // 一番古いのを削除
        while (getSize() >= 1000) {
            InfoKey tmp = searchLocal("").get(0);
            remove(tmp);
        }
        synchronized (keys) {
            keys.add(key);
        }
        AppMain.debug("O キー追加 <" + key.getFilename() + ">");
    }

    /**
     * キーを削除
     */
    public void remove(InfoKey key) {
        synchronized (keys) {
            keys.remove(key);
        }
        AppMain.debug("O キー削除 <" + key.getFilename() + ">");
    }
}
