/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package pp4u_manage;

import pp4u.*;
import java.io.IOException;
import java.util.ArrayList;
import pp4u_packet.Packet;

/**
 * リンク管理クラス
 * @author tondol
 */
public class ManageLinks implements Runnable {

    static private ManageLinks instance = null;
    private ArrayList<LinkSearch> links = null;
    private volatile Thread thread = null;

    /**
     * リンクリスト初期化
     */
    private ManageLinks() {
        links = new ArrayList<LinkSearch>();
    }

    /**
     * インスタンスを取得
     */
    static synchronized public ManageLinks getInstance() {
        if (instance == null) {
            instance = new ManageLinks();
        }
        return instance;
    }

    /**
     * リンク管理起動
     */
    public synchronized void start() {
        if (thread == null) {
            thread = new Thread(this);
            thread.start();
        }
    }

    /**
     * リンク管理停止
     */
    public synchronized void stop() {
        if (thread == null) {
            return;
        } else {
            thread.interrupt();
        }

        ManageNodes manager = ManageNodes.getInstance();
        LinkSearch[] arrayLinks = null;
        synchronized (links) {
            arrayLinks = links.toArray(new LinkSearch[0]);
        }
        for (LinkSearch link : arrayLinks) {
            move(link);
        }
        manager.publish();
    }

    /**
     * リンク管理スレッド
     */
    public void run() {
        try {
            while (!thread.isInterrupted()) {
                LinkSearch[] arrayLinks = null;
                synchronized (links) {
                    arrayLinks = links.toArray(new LinkSearch[0]);
                }
                for (LinkSearch link : arrayLinks) {
                    if (link.getPriority() == -1) {
                        move(link);
                    }
                }
                ManageNodes.getInstance().publish();
                Thread.sleep(1 * 1000);
            }
        } catch (Exception ex) {
        } finally {
            thread = null;
        }
    }

    /**
     * ノードIDに合致するリンクを返す
     */
    public LinkSearch search(String others) {
        synchronized (links) {
            for (LinkSearch link : links) {
                if (link.getPriority() == -1) {
                    continue;
                } else if (link.getNodeID().equals(others)) {
                    return link;
                }
            }
        }
        return null;
    }

    /**
     * すべての検索リンクに送信(exceptを除く)
     */
    public void broadcast(Packet packet, LinkSearch except) throws IOException {
        synchronized (links) {
            for (LinkSearch link : links) {
                if (link.equals(except)) {
                    continue;
                }
                if (link.getPriority() == -1) {
                    continue;
                }
                link.send(packet);
            }
        }
    }

    /**
     * 検索リンクを返す
     */
    public int getSize() {
        synchronized (links) {
            return links.size();
        }
    }

    /**
     * リンクを切断してノードに移動
     */
    public void move(LinkSearch link) {
        ManageNodes manager = ManageNodes.getInstance();
        remove(link);
        manager.add(link.getGrobalAddr());
    }

    /**
     * リンクを追加 - ノードを消す必要があればtrue
     */
    public boolean add(LinkSearch link) {
        // 既に5つ以上あったら追加しない
        if (getSize() >= 5) {
            return false;
        }
        // 重複チェック
        LinkSearch matched = search(link.getNodeID());
        if (matched != null) {
            String s1 = link.getLinkID();
            String s2 = matched.getLinkID();
            if (s1.compareTo(s2) > 0) {
                remove(matched);
            } else {
                return true;
            }
        }
        // 追加
        synchronized (links) {
            links.add(link);
        }
        AppMain.debug("O 検索リンク追加 <" + link.getGrobalAddr() + ">");
        return true;
    }

    /**
     * リンクを削除
     */
    public void remove(LinkSearch link) {
        link.stop();
        synchronized (links) {
            links.remove(link);
        }
        AppMain.debug("X 検索リンク削除 <" + link.getGrobalAddr() + ">");
    }

    //////////////////////////////////////////////
    // TableModelの実装もね - 実態はManageNodes
    //////////////////////////////////////////////
    public int getRowCount() {
        return getSize();
    }

    public Object getValueAt(int rowIndex, int columnIndex) {
        Object obj = null;
        LinkSearch node = null;
        synchronized (links) {
            node = links.get(rowIndex);
        }

        switch (columnIndex) {
            case 0:
                obj = "検索リンク(ﾟ∀ﾟ)";
                break;
            case 1:
                if (node.isBehind()) {
                    obj = "NAT経由";
                } else {
                    obj = "直接接続";
                }
                break;
            case 2:
                obj = node.getGrobalAddr();
                break;
            case 3:
                obj = node.getPriority();
                break;
        }
        return obj;
    }
}
