/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package pp4u_manage;

import pp4u.*;
import pp4u_packet.PacketPing;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.net.InetSocketAddress;
import java.net.ServerSocket;
import java.net.Socket;

/**
 * ソケット管理クラス
 * @author tondol
 */
public class ManageListener implements Runnable {

    static private ManageListener instance = null;
    private volatile ServerSocket listener = null;
    private volatile Thread thread = null;

    /**
     * コンストラクタなんて飾りです
     */
    private ManageListener() {
    }

    static synchronized public ManageListener getInstance() {
        if (instance == null) {
            instance = new ManageListener();
        }
        return instance;
    }

    public synchronized void start() throws Exception {
        InfoNode localhost = AppMain.getLocalhost();
        try {
            // 待ち受け開始
            listener = new ServerSocket();
            listener.bind(new InetSocketAddress(localhost.getPort()));
            thread = new Thread(this);
            thread.start();

            // デバッグ表示
            AppMain.debug("O 待ち受け開始 <" + localhost.getPort() + ">");
        } catch (Exception ex) {
            AppMain.debug("X 待ち受け失敗 <" + localhost.getPort() + ">");
            AppMain.debug("@ " + ex);
            stop();
            throw ex;
        }
    }

    public synchronized void stop() {
        try {
            if (thread != null) {
                thread.interrupt();
            }
            if (listener != null) {
                listener.close();
            }
        } catch (Exception ex) {
        } finally {
            AppMain.debug("O 待ち受け停止");
        }
    }

    public void run() {
        try {
            while (!listener.isClosed()) {
                Socket socket = listener.accept();
                callbacks(socket);
            }
        } catch (Exception ex) {
            if (listener != null) {
                try {
                    listener.close();
                } catch (IOException iex) {
                }
            }
        } finally {
            thread = null;
            listener = null;
        }
    }

    private void callbacks(Socket socket) {
        try {
            ObjectInputStream in = new ObjectInputStream(socket.getInputStream());
            PacketPing ping = (PacketPing) in.readObject();

            // リンク受信
            if (ping.getType() == PacketPing.SEARCH) {
                ManageLinks manager = ManageLinks.getInstance();
                LinkSearch node = new LinkSearch(socket, in, ping);
                manager.add(node);
                AppMain.debug("# 検索リンク受信");
            } else if (ping.getType() == PacketPing.TRANSFER) {
                ManageTasks manager = ManageTasks.getInstance();
                TaskUpload task = new TaskUpload(socket, in, ping);
                manager.add(task);
                AppMain.debug("# 送信タスク受信");
            }
        } catch (Exception ex) {
            try {
                socket.close();
                socket = null;
            } catch (IOException iex) {
            }
        }
    }
}
