/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package pp4u_manage;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import javax.swing.table.AbstractTableModel;
import pp4u.AppMain;
import pp4u.InfoNode;
import pp4u.LinkSearch;

/**
 * ノード管理クラス
 * @author tondol
 */
public class ManageNodes extends AbstractTableModel implements Runnable {

    static private ManageNodes instance = null;
    private ArrayList<InfoNode> nodes = null;
    private volatile Thread thread = null;

    /**
     * コンストラクタなんて飾りです
     */
    private ManageNodes() {
        nodes = new ArrayList<InfoNode>();
    }

    /**
     * インスタンスを取得
     */
    static synchronized public ManageNodes getInstance() {
        if (instance == null) {
            instance = new ManageNodes();
        }
        return instance;
    }

    /**
     * ノードリスト読み込み
     */
    public void load() {
        try {
            // ファイルを開く
            FileReader fst = new FileReader("pp4unode.txt");
            BufferedReader ist = new BufferedReader(fst);
            String line = null;

            // 終端まで読み込む
            while ((line = ist.readLine()) != null) {
                InfoNode info = new InfoNode(line);
                add(info);
            }
            ist.close();

            // デバッグ表示
            AppMain.debug("O ノードリスト読込成功");
        } catch (IOException iex) {
            AppMain.debug("X ノードリスト読込失敗");
            AppMain.debug("@ " + iex);
        }
    }

    /**
     * ノード管理起動
     */
    public synchronized void start() {
        if (thread != null) {
            return;
        } else {
            thread = new Thread(this);
            thread.start();
        }
    }

    /**
     * ノード管理停止
     */
    public synchronized void stop() {
        try {
            // ファイルを開く
            FileWriter fst = new FileWriter("pp4unode.txt");
            PrintWriter ost = new PrintWriter(fst);

            // ノードリストを書き込んで切断
            for (InfoNode node : nodes) {
                String line = node.toString();
                ost.println(line);
            }
            ost.flush();
            ost.close();

            // デバッグ表示
            AppMain.debug("O ノードリスト書込成功");
        } catch (Exception ex) {
            AppMain.debug("X ノードリスト書込失敗");
            AppMain.debug("@ " + ex);
        } finally {
            if (thread != null) {
                thread.interrupt();
            }
        }
    }

    /**
     * ノード管理スレッド
     */
    public void run() {
        try {
            while (!thread.isInterrupted()) {
                callbacks();
                publish();
                Thread.sleep(10 * 1000);
            }
        } catch (Exception ex) {
        } finally {
            thread = null;
        }
    }

    /**
     * コールバックメソッド
     */
    private void callbacks() {
        ManageLinks manager = ManageLinks.getInstance();
        InfoNode[] arrayNodes = null;
        synchronized (nodes) {
            arrayNodes = nodes.toArray(new InfoNode[0]);
        }
        // 検索リンク数を5個に調整
        for (InfoNode node : arrayNodes) {
            if (manager.getSize() >= 5) {
                break;
            }
            move(node);
        }
    }

    public int getSize() {
        synchronized (nodes) {
            return nodes.size();
        }
    }

    /**
     * 接続してリンクリストに移動
     */
    public void move(InfoNode node) {
        ManageLinks manager = ManageLinks.getInstance();
        LinkSearch link = null;
        try {
            link = new LinkSearch(node);
            if (manager.add(link)) {
                remove(node);
            }
        } catch (Exception ex) {
            return;
        }
    }

    /**
     * ノードを追加 - 重複を許可しない
     */
    public void add(InfoNode node) {
        synchronized (nodes) {
            if (nodes.contains(node)) {
                return;
            }
        }
        // 既に100個以上あったら
        while (getSize() >= 100) {
            remove(nodes.get(0));
        }
        // 追加
        synchronized (nodes) {
            nodes.add(node);
        }
        publish();
        AppMain.debug("O ノード追加 <" + node + ">");
    }

    /**
     * ノードを削除
     */
    public void remove(InfoNode node) {
        synchronized (nodes) {
            nodes.remove(node);
        }
        publish();
        AppMain.debug("X ノード削除 <" + node + ">");
    }

    //////////////////////////////////////////////
    // TableModelの実装もね - ManageLinksに分散
    //////////////////////////////////////////////
    /**
     * 外部からテーブルを更新
     */
    public void publish() {
        fireTableDataChanged();
    }

    @Override
    public int getColumnCount() {
        return 4;
    }

    @Override
    public String getColumnName(int column) {
        String str = null;
        switch (column) {
            case 0:
                str = "状態";
                break;
            case 1:
                str = "種別";
                break;
            case 2:
                str = "接続先";
                break;
            case 3:
                str = "優先度";
                break;
        }
        return str;
    }

    @Override
    public int getRowCount() {
        ManageLinks manager = ManageLinks.getInstance();
        return manager.getRowCount() + getSize();
    }

    @Override
    public Object getValueAt(int rowIndex, int columnIndex) {
        // リンクの値を取得
        ManageLinks manager = ManageLinks.getInstance();
        if (rowIndex < manager.getRowCount()) {
            return manager.getValueAt(rowIndex, columnIndex);
        } else {
            rowIndex -= manager.getRowCount();
        }

        Object obj = null;
        InfoNode node = null;
        synchronized (nodes) {
            node = nodes.get(rowIndex);
        }

        // ノードの値を取得
        switch (columnIndex) {
            case 0:
                obj = "待機(´・ω・｀)";
                break;
            case 1:
                obj = "未接続";
                break;
            case 2:
                obj = node;
                break;
            case 3:
                obj = "未接続";
                break;
        }
        return obj;
    }
}

