/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package pp4u_manage;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import javax.swing.table.AbstractTableModel;
import pp4u.AppMain;
import pp4u.InfoKey;
import pp4u_packet.PacketQuery;

/**
 * 検索リスト管理
 * @author tondol
 */
public class ManageSearch extends AbstractTableModel implements Runnable {

    static private ManageSearch instance = null;
    private String currentWord = null;
    private List<InfoKey> currentKeys = null;
    private volatile Thread thread = null;

    /**
     * 初期化
     */
    private ManageSearch() {
        currentKeys = new ArrayList<InfoKey>();
        currentWord = "";
    }

    /**
     * インスタンス化
     */
    static synchronized public ManageSearch getInstance() {
        if (instance == null) {
            instance = new ManageSearch();
        }
        return instance;
    }

    /**
     * 検索ワードを指定してスレッド起動
     */
    public synchronized void start(String word) {
        if (thread != null) {
            return;
        }

        ManageKeys manager = ManageKeys.getInstance();
        currentWord = word;
        currentKeys = manager.searchAll(currentWord);
        thread = new Thread(this);
        thread.start();
        fireTableDataChanged();
        //AppMain.debug("O 検索タスク開始 <" + currentWord + ">");
    }

    /**
     * スレッド停止
     */
    public synchronized void stop() {
        if (thread == null) {
            return;
        }

        thread.interrupt();
        thread = null;
        currentWord = "";
        currentKeys.clear();
        fireTableDataChanged();
        //AppMain.debug("O 検索タスク終了 <" + currentWord + ">");
    }

    /**
     * クエリ発行
     */
    public void run() {
        try {
            while (!thread.isInterrupted()) {
                ManageLinks manager = ManageLinks.getInstance();
                PacketQuery query = new PacketQuery(currentWord);
                if (currentWord.length() != 0) {
                    try {
                        manager.broadcast(query, null);
                    } catch (IOException iex) {
                    }
                }
                Thread.sleep(30 * 1000);
            }
        } catch (Exception ex) {
        } finally {
            thread = null;
        }
    }

    /**
     * キーを読み込む
     */
    public void search(String word) {
        if (!currentWord.equals(word)) {
            if (currentWord.length() != 0) {
                stop();
            }
            if (word.length() != 0) {
                start(word);
            }
        } else if (word.length() != 0) {
            ManageKeys manager = ManageKeys.getInstance();
            currentKeys = manager.searchAll(currentWord);
            fireTableDataChanged();
        }
    }
    
    public InfoKey getKey(int index) {
        return currentKeys.get(index);
    }

    //////////////////////////////////////////////
    // TableModelの実装もね
    //////////////////////////////////////////////
    @Override
    public int getColumnCount() {
        return 5;
    }

    @Override
    public String getColumnName(int column) {
        String ret = null;
        switch (column) {
            case 0:
                ret = "ファイル名";
                break;
            case 1:
                ret = "サイズ";
                break;
            case 2:
                ret = "種別";
                break;
            case 3:
                ret = "送信元";
                break;
            case 4:
                ret = "ハッシュ";
                break;
        }
        return ret;
    }

    public int getRowCount() {
        return currentKeys.size();
    }

    public Object getValueAt(int rowIndex, int columnIndex) {
        Object obj = null;
        InfoKey key = currentKeys.get(rowIndex);
        switch (columnIndex) {
            case 0:
                obj = key.getFilename();
                break;
            case 1:
                obj = key.getSize();
                break;
            case 2:
                if (key.isLocalKey()) {
                    obj = "保持キー";
                } else {
                    obj = "仮想キー";
                }
                break;
            case 3:
                obj = key.getNode();
                break;
            case 4:
                obj = key.getDigest();
                break;
        }
        return obj;
    }
}
